@@ -101,7 +101,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: idle_scan.cc 2813 2005-08-11 05:07:01Z fyodor $ */
+/* $Id: idle_scan.cc 3224 2006-03-25 23:56:48Z fyodor $ */
 
 #include "idle_scan.h"
 #include "scan_engine.h"
@@ -154,7 +154,7 @@ struct idle_proxy_info {
    Proxy timing is adjusted, but proxy->latestid is NOT ADJUSTED --
    you'll have to do that yourself.   Probes_sent is set to the number
    of probe packets sent during execution */
-int ipid_proxy_probe(struct idle_proxy_info *proxy, int *probes_sent,
+static int ipid_proxy_probe(struct idle_proxy_info *proxy, int *probes_sent,
 		     int *probes_rcvd) {
   struct timeval tv_end;
   int tries = 0;
@@ -191,7 +191,7 @@ int ipid_proxy_probe(struct idle_proxy_info *proxy, int *probes_sent,
 		 proxy->probe_port,
 		 seq_base + (packet_send_count++ * 500) + 1, ack, 
 		 TH_SYN|TH_ACK, 0, 
-		 NULL, 0, NULL, 0);
+		 (u8 *) "\x02\x04\x05\xb4", 4, NULL, 0);
     sent++;
     tries++;
 
@@ -248,7 +248,7 @@ int ipid_proxy_probe(struct idle_proxy_info *proxy, int *probes_sent,
    one, assuming the given IPID Sequencing class.  Returns -1 if the
    distance cannot be determined */
 
-int ipid_distance(int seqclass , u16 startid, u16 endid) {
+static int ipid_distance(int seqclass , u16 startid, u16 endid) {
   if (seqclass == IPID_SEQ_INCR)
     return endid - startid;
   
@@ -263,13 +263,23 @@ int ipid_distance(int seqclass , u16 startid, u16 endid) {
 
 }
 
+static void initialize_proxy_struct(struct idle_proxy_info *proxy) {
+  proxy->seqclass = proxy->latestid = proxy->probe_port = 0;
+  proxy->max_groupsz = proxy->min_groupsz = 0;
+  proxy->current_groupsz = 0;
+  proxy->senddelay = 0; 
+  proxy->max_senddelay = 0;
+  proxy->pd = NULL;
+  proxy->rawsd = -1;
+  proxy->ethptr = NULL;
+}
 
 /* takes a proxy name/IP, resolves it if neccessary, tests it for IPID
    suitability, and fills out an idle_proxy_info structure.  If the
    proxy is determined to be unsuitable, the function whines and exits
    the program */
 #define NUM_IPID_PROBES 6
-void initialize_idleproxy(struct idle_proxy_info *proxy, char *proxyName,
+static void initialize_idleproxy(struct idle_proxy_info *proxy, char *proxyName,
 			  const struct in_addr *first_target) {
   int probes_sent = 0, probes_returned = 0;
   int hardtimeout = 9000000; /* Generally don't wait more than 9 secs total */
@@ -301,7 +311,7 @@ void initialize_idleproxy(struct idle_proxy_info *proxy, char *proxyName,
 
   for(i=0; i < NUM_IPID_PROBES; i++) probe_returned[i] = 0;
 
-  memset(proxy, 0, sizeof(*proxy));
+  initialize_proxy_struct(proxy);
   initialize_timeout_info(&proxy->host.to);
 
   proxy->max_groupsz = (o.max_parallelism)? o.max_parallelism : 100;
@@ -361,12 +371,14 @@ void initialize_idleproxy(struct idle_proxy_info *proxy, char *proxyName,
 	    __FUNCTION__);
     memcpy(proxy->eth.srcmac, proxy->host.SrcMACAddress(), 6);
     memcpy(proxy->eth.dstmac, proxy->host.NextHopMACAddress(), 6);
-    proxy->eth.ethsd = eth_open(proxy->host.deviceName());
+    proxy->eth.ethsd = eth_open_cached(proxy->host.deviceName());
+    if (proxy->eth.ethsd == NULL)
+      fatal("%s: Failed to open ethernet device (%s)", __FUNCTION__, proxy->host.deviceName());
     proxy->rawsd = -1;
     proxy->ethptr = &proxy->eth;
   } else {
     if ((proxy->rawsd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0 )
-      pfatal("socket trobles in get_fingerprint");
+      pfatal("socket trobles in %s", __FUNCTION__);
     unblock_socket(proxy->rawsd);
     broadcast_socket(proxy->rawsd);
 #ifndef WIN32
@@ -407,8 +419,8 @@ void initialize_idleproxy(struct idle_proxy_info *proxy, char *proxyName,
     send_tcp_raw(proxy->rawsd, proxy->ethptr, proxy->host.v4sourceip(), 
 		 proxy->host.v4hostip(), o.ttl, 
 		 o.magic_port + probes_sent + 1, proxy->probe_port, 
-		 sequence_base + probes_sent + 1, 0, TH_SYN|TH_ACK, 
-		 ack, NULL, 0, NULL, 0);
+		 sequence_base + probes_sent + 1, ack, TH_SYN|TH_ACK, 
+		 0, (u8 *) "\x02\x04\x05\xb4", 4, NULL, 0);
     gettimeofday(&probe_send_times[probes_sent], NULL);
     probes_sent++;
 
@@ -515,7 +527,7 @@ void initialize_idleproxy(struct idle_proxy_info *proxy, char *proxyName,
 		   proxy->host.v4hostip(), 
 		   o.ttl, o.magic_port, proxy->probe_port, 
 		   sequence_base + probes_sent + 1, 0, TH_SYN|TH_ACK, 
-		   ack, NULL, 0, NULL, 0);
+		   ack, (u8 *) "\x02\x04\x05\xb4", 4, NULL, 0);
 
     }
 
@@ -548,7 +560,7 @@ void initialize_idleproxy(struct idle_proxy_info *proxy, char *proxyName,
    count of 'testcount' while the 'realcount' is as given.  If the
    testcount was correct, timing is made more aggressive, while it is
    slowed down in the case of an error */
-void adjust_idle_timing(struct idle_proxy_info *proxy, 
+static void adjust_idle_timing(struct idle_proxy_info *proxy, 
 			Target *target, int testcount, 
 			int realcount) {
 
@@ -610,7 +622,7 @@ void adjust_idle_timing(struct idle_proxy_info *proxy,
    They can be NULL if you don't want to use them.  The purpose is for
    timing adjustments if the numbers turn out to be accurate */
 
-int idlescan_countopen2(struct idle_proxy_info *proxy, 
+static int idlescan_countopen2(struct idle_proxy_info *proxy, 
 			Target *target, u16 *ports, int numports,
 			struct timeval *sent_time, struct timeval *rcv_time) 
 {
@@ -654,7 +666,9 @@ int idlescan_countopen2(struct idle_proxy_info *proxy,
 	    __FUNCTION__);
     memcpy(eth.srcmac, target->SrcMACAddress(), 6);
     memcpy(eth.dstmac, target->NextHopMACAddress(), 6);
-    eth.ethsd = eth_open(target->deviceName());
+    eth.ethsd = eth_open_cached(target->deviceName());
+    if (eth.ethsd == NULL)
+      fatal("%s: Failed to open ethernet device (%s)", __FUNCTION__, target->deviceName());
   } else eth.ethsd = NULL;
 
   /* I start by sending out the SYN pr0bez */
@@ -669,7 +683,7 @@ int idlescan_countopen2(struct idle_proxy_info *proxy,
     send_tcp_raw(proxy->rawsd, eth.ethsd? &eth : NULL, proxy->host.v4hostip(), 
 		 target->v4hostip(),
 		 o.ttl, proxy->probe_port, ports[pr0be], seq, 0, TH_SYN, 0,
-		 NULL, 0, o.extra_payload, o.extra_payload_length);
+		 (u8 *) "\x02\x04\x05\xb4", 4, o.extra_payload, o.extra_payload_length);
   }
   gettimeofday(&end, NULL);
 
@@ -754,7 +768,7 @@ int idlescan_countopen2(struct idle_proxy_info *proxy,
     if (rcv_time) *rcv_time = latestchange;
   }
   if (newipid > 0) proxy->latestid = newipid;
-  if (eth.ethsd) { eth_close(eth.ethsd); eth.ethsd = NULL; }
+  if (eth.ethsd) { eth.ethsd = NULL; } /* don't need to close it due to caching */
   return openports;
 }
 
@@ -763,7 +777,7 @@ int idlescan_countopen2(struct idle_proxy_info *proxy,
 /* The job of this function is to use the Idlescan technique to count
    the number of open ports in the given list.  Under the covers, this
    function just farms out the hard work to another function */
-int idlescan_countopen(struct idle_proxy_info *proxy, 
+static int idlescan_countopen(struct idle_proxy_info *proxy, 
 		       Target *target, u16 *ports, int numports,
 		       struct timeval *sent_time, struct timeval *rcv_time) {
   int tries = 0;
@@ -804,7 +818,7 @@ int idlescan_countopen(struct idle_proxy_info *proxy,
 /* Recursively Idlescans scans a group of ports using a depth-first
    divide-and-conquer strategy to find the open one(s) */
 
-int idle_treescan(struct idle_proxy_info *proxy, Target *target,
+static int idle_treescan(struct idle_proxy_info *proxy, Target *target,
 		 u16 *ports, int numports, int expectedopen) {
 
   int firstHalfSz = (numports + 1)/2;
