@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.h 2820 2005-08-24 19:22:11Z fyodor $ */
+/* $Id: tcpip.h 3224 2006-03-25 23:56:48Z fyodor $ */
 
 
 #ifndef TCPIP_H
@@ -198,7 +198,6 @@ void *realloc();
 #include <unistd.h>
 #endif
 #include <fcntl.h>
-#include <sys/socket.h>
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -258,6 +257,11 @@ typedef enum { devt_ethernet, devt_loopback, devt_p2p, devt_other  } devtype;
 #define TH_CWR        0x80
 #endif
 
+#ifndef INET_ADDRSTRLEN
+#define INET_ADDRSTRLEN 16
+#endif
+
+
 /* Used for tracing all packets sent or received (eg the
    --packet_trace option) */
 class PacketTrace {
@@ -286,7 +290,7 @@ class PacketTrace {
      direction must be PacketTrace::SENT or PacketTrace::RCVD .
      Optional 'now' argument makes this function slightly more
      efficient by avoiding a gettimeofday() call. */
-  static void PacketTrace::traceArp(pdirection pdir, const u8 *frame, u32 len,
+  static void traceArp(pdirection pdir, const u8 *frame, u32 len,
 				    struct timeval *now);
 };
 
@@ -323,7 +327,7 @@ struct route_nfo {
   struct interface_info ii;
 
 /* true if the target is directly connected on the network (no routing
-   required. */
+   required). */
   bool direct_connect; 
 
 /* This is the source address that should be used by the packets.  It
@@ -545,8 +549,10 @@ int resolve(char *hostname, struct in_addr *ip);
    source address and interface necessary to route to this address.
    If no route is found, false is returned and rnfo is undefined.  If
    a route is found, true is returned and rnfo is filled in with all
-   of the routing details */
+   of the routing details.  This function takes into account -S and -e
+   options set by user (o.spoofsource, o.device) */
 bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo);
+
 /* Determines what interface packets destined to 'dest' should be
    routed through.  It can also discover the appropriate next hop (if
    any) for ethernet routing.  If direct_connect is passed in, it will
@@ -622,11 +628,6 @@ u8 *build_ip_raw(const struct in_addr *source, const struct in_addr *victim,
 int send_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, 
 		   unsigned int packetlen);
 
-/* Create and send all fragments of the pre-built packet */
-/* mtu = MTU - ipv4_headerlen */
-int send_frag_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, 
-			unsigned int packetlen, unsigned int mtu);
-
 /* Decoy versions of the raw packet sending functions ... */
 int send_tcp_raw_decoys( int sd, struct eth_nfo *eth, 
 			 const struct in_addr *victim, int ttl,
@@ -649,15 +650,19 @@ pcap_t *my_pcap_open_live(const char *device, int snaplen, int promisc,
 // invalid (Windows and Amiga), readip_pcap returns the time you called it.
 bool pcap_recv_timeval_valid();
 
-/* Returns a buffer of ASCII information about a packet that may look
-   like "TCP 127.0.0.1:50923 > 127.0.0.1:3 S ttl=61 id=39516 iplen=40
-   seq=625950769" or "ICMP PING (0/1) ttl=61 id=39516 iplen=40".
-   Since this is a static buffer, don't use threads or call twice
-   within (say) printf().  And certainly don't try to free() it!  The
-   returned buffer is NUL-terminated */
-const char *ippackethdrinfo(const u8 *packet, u32 len);
-/* Shows the most important fields of an IP packet (including dissecting TCP/UDP headers.  packet should point to the beginning of the IP header  */
-void readippacket(const u8 *packet, int readdata);
+/* A simple function that caches the eth_t from dnet for one device,
+   to avoid opening, closing, and re-opening it thousands of tims.  If
+   you give a different device, this function will close the first
+   one.  Thus this should never be used by programs that need to deal
+   with multiple devices at once.  In addition, you MUST NEVER
+   eth_close() A DEVICE OBTAINED FROM THIS FUNCTION.  Instead, you can
+   call eth_close_cached() to close whichever device (if any) is
+   cached.  Returns NULL if it fails to open the device. */
+eth_t *eth_open_cached(const char *device);
+
+/* See the description for eth_open_cached */
+void eth_close_cached();
+
 /* A simple function I wrote to help in debugging, shows the important fields
    of a TCP packet*/
 int readtcppacket(const u8 *packet, int readdata);
@@ -667,8 +672,16 @@ int readudppacket(const u8 *packet, int readdata);
 int ipaddr2devname( char *dev, const struct in_addr *addr );
 /* And vice versa */
 int devname2ipaddr(char *dev, struct in_addr *addr);
-/* Where the above 2 functions get their info */
+/* Looks for an interface assigned to the given IP (ss), and returns
+   the interface_info for the first one found.  If non found, returns NULL */
+struct interface_info *getInterfaceByIP(struct sockaddr_storage *ss);
+/* Looks for an interface with the given name (iname), and returns the
+   corresponding interface_info if found.  Will accept a match of
+   devname or devfullname.  Returns NULL if none found */
+struct interface_info *getInterfaceByName(char *iname);
+/* Where the above 4 functions get their info */
 struct interface_info *getinterfaces(int *howmany);
+
 /* Parse the system routing table, converting each route into a
    sys_route entry.  Returns an array of sys_routes.  numroutes is set
    to the number of routes in the array.  The routing table is only
@@ -711,8 +724,6 @@ bool setTargetNextHopMAC(Target *target);
 
 int islocalhost(const struct in_addr * const addr);
 int unblock_socket(int sd);
-int Sendto(char *functionname, int sd, const unsigned char *packet, int len, 
-	   unsigned int flags, struct sockaddr *to, int tolen);
 
 // Takes a protocol number like IPPROTO_TCP, IPPROTO_UDP, or
 // IPPROTO_TCP and returns a ascii representation (or "unknown" if it
@@ -729,19 +740,6 @@ int get_link_offset(char *device);
 char *readip_pcap(pcap_t *pd, unsigned int *len, long to_usec, 
 		  struct timeval *rcvdtime, struct link_header *linknfo);
 
-/* A trivial functon that maintains a cache of IP to MAC Address
-   entries.  If the command is ARPCACHE_GEt, this func looks for the
-   IPv4 address in ss and fills in the 'mac' parameter and returns
-   true if it is found.  Otherwise (not found), the function returns
-   false.  If the command is ARPCACHE_SET, the function adds an entry
-   with the given ip (ss) and mac address.  An existing entry for the
-   IP ss will be overwritten with the new MAC address.  true is always
-   returned for the set command. */
-#define ARPCACHE_GET 1
-#define ARPCACHE_SET 2
-bool NmapArpCache(int command, struct sockaddr_storage *ss, u8 *mac);
-
-
 /* Attempts to read one IPv4/Ethernet ARP reply packet from the pcap
    descriptor pd.  If it receives one, fills in sendermac (must pass
    in 6 bytes), senderIP, and rcvdtime (can be NULL if you don't care)
@@ -752,15 +750,6 @@ bool NmapArpCache(int command, struct sockaddr_storage *ss, u8 *mac);
 int read_arp_reply_pcap(pcap_t *pd, u8 *sendermac, struct in_addr *senderIP,
 		       long to_usec, struct timeval *rcvdtime);
 
-/* Issues an ARP request for the MAC of targetss (which will be placed
-   in targetmac if obtained) from the source IP (srcip) and source mac
-   (srcmac) given.  "The request is ussued using device dev to the
-   broadcast MAC address.  The transmission is attempted up to 3
-   times.  If none of these elicit a response, false will be returned.
-   If the mac is determined, true is returned. */
-bool doArp(const char *dev, u8 *srcmac, struct sockaddr_storage *srcip, 
-	   struct sockaddr_storage *targetip, u8 *targetmac);
-
 #ifndef HAVE_INET_ATON
 int inet_aton(register const char *, struct in_addr *);
 #endif
